package ai.huisuan.back.service.impl;

import ai.huisuan.back.entity.DialogueSessionInfo;
import ai.huisuan.back.entity.HistoryRecordInfo;
import ai.huisuan.back.model.*;
import ai.huisuan.back.repository.DialogueSessionsRepository;
import ai.huisuan.back.repository.HistoryRecordsRepository;
import ai.huisuan.back.service.ChatService;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import okhttp3.*;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.servlet.mvc.method.annotation.SseEmitter;

import java.util.ArrayList;
import java.util.List;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.time.Duration;
import java.time.Instant;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Service
public class ChatServiceImpl implements ChatService {

    // @Value("${fwwb.server.address}")
    private String ollamaHost="127.0.0.1";

    @Value("${ollama.server.port}")
    private int ollamaPort;

    private final ObjectMapper objectMapper;
    private final DialogueSessionsRepository dialogueSessionsRepository;
    private final HistoryRecordsRepository historyRecordsRepository;
    private final OkHttpClient client;
    private final ExecutorService executor = Executors.newCachedThreadPool();

    private static final Duration CONNECT_TIMEOUT = Duration.ofSeconds(300);
    private static final Duration READ_TIMEOUT = Duration.ofSeconds(600);
    private static final Duration WRITE_TIMEOUT = Duration.ofSeconds(300);
    private static final Duration SSE_TIMEOUT = Duration.ofMinutes(10);

    public ChatServiceImpl(ObjectMapper objectMapper,
                           DialogueSessionsRepository dialogueSessionsRepository,
                           HistoryRecordsRepository historyRecordsRepository) {
        this.objectMapper = objectMapper;
        this.dialogueSessionsRepository = dialogueSessionsRepository;
        this.historyRecordsRepository = historyRecordsRepository;
        this.client = new OkHttpClient.Builder()
                .connectTimeout(CONNECT_TIMEOUT)
                .readTimeout(READ_TIMEOUT)
                .writeTimeout(WRITE_TIMEOUT)
                .build();
    }

    @Override
    public Request buildOllamaRequest(ChatRequest requestDTO,boolean stream) throws JsonProcessingException {
        // 保存数据
        HistoryRecordInfo historyRecordInfo = new HistoryRecordInfo();
        historyRecordInfo.setContent(requestDTO.getQuestion().getContent());
        historyRecordInfo.setRole(requestDTO.getQuestion().getRole());
        historyRecordInfo.setTimestamp(Instant.now());
        DialogueSessionInfo dialogueSessionInfo = dialogueSessionsRepository.findById(requestDTO.getSessionId()).orElse(null);
        historyRecordInfo.setSession(dialogueSessionInfo);
        historyRecordsRepository.save(historyRecordInfo);

        // 构建请求体
        List<OllamaMessage> ollamaMessages = new ArrayList<>();

        List<Message> messages = requestDTO.getHistory().getMessages();
        for (Message message : messages) {
            OllamaMessage ollamaMessage = new OllamaMessage(message.getRole(), message.getContent());
            ollamaMessages.add(ollamaMessage);
        }

        ollamaMessages.add(new OllamaMessage(requestDTO.getQuestion().getRole(),requestDTO.getQuestion().getContent()));

        System.out.println(ollamaMessages);

        OllamaChatRequest request = new OllamaChatRequest(
                requestDTO.getModel(),
                ollamaMessages,
                stream
        );

        return new Request.Builder()
                .url(String.format("http://%s:%d/api/chat", ollamaHost, ollamaPort))
                .post(RequestBody.create(
                        objectMapper.writeValueAsString(request),
                        MediaType.parse("application/json")
                ))
                .build();
    }


    @Override
    public ChatResponse parseResponse(String responseBody,ChatRequest chatRequest) {

            // id之后设置
            Message message= new Message(1,"model",responseBody, Instant.now());
            History history =chatRequest.getHistory();
            history.getMessages().add(chatRequest.getQuestion());
            history.getMessages().add(new Message());
            ChatResponse responseDTO = new ChatResponse();
            responseDTO.setAnswer(message);
            responseDTO.setModel(message.getRole());
            responseDTO.setCreatedAt(message.getTimestamp());
            responseDTO.setHistory(history);

            // 保存数据到数据库
            HistoryRecordInfo historyRecordInfo = new HistoryRecordInfo();
            historyRecordInfo.setContent(responseBody);
            historyRecordInfo.setRole(responseDTO.getAnswer().getRole());
            historyRecordInfo.setTimestamp(Instant.now());
            DialogueSessionInfo dialogueSessionInfo = dialogueSessionsRepository.findById(chatRequest.getSessionId()).orElse(null);
            historyRecordInfo.setSession(dialogueSessionInfo);
            historyRecordsRepository.save(historyRecordInfo);

            // 返回结果
            return responseDTO;
    }



    @Override
    public ChatResponse completions(ChatRequest requestDTO) throws IOException {
            Request request = buildOllamaRequest(requestDTO,false);
            Response response = client.newCall(request).execute();
            System.out.println(response);
            return parseResponse(response.body().string(),requestDTO);
    }

    @Override
    public SseEmitter streamChat(ChatRequest requestDTO) {
        SseEmitter emitter = new SseEmitter(SSE_TIMEOUT.toMillis());
        executor.execute(() -> {
            try {
                executeStreamingRequest(emitter, requestDTO);
            } catch (Exception e) {
                sendError(emitter, "Initialization failed: " + e.getMessage());
            }
        });
        return emitter;
    }


    private void executeStreamingRequest(SseEmitter emitter, ChatRequest requestDTO) throws JsonProcessingException {
        Request request = buildOllamaRequest(requestDTO,true);
        client.newCall(request).enqueue(new Callback() {
            @Override
            public void onResponse(@NotNull Call call, @NotNull Response response) throws IOException {
                ResponseBody body = response.body();
                System.out.println(body);

                BufferedReader reader = new BufferedReader(new InputStreamReader(body.byteStream()));

                StringBuilder fullResponse = new StringBuilder();
                String line;
                while ((line=reader.readLine()) != null) {
                    OllamaChatResponse chunk = objectMapper.readValue(line, OllamaChatResponse.class);
                    System.out.println(chunk);
                    fullResponse.append(chunk.getMessage().getContent());
                    processChunk(emitter, chunk);
                    if (chunk.isDone()) {
                        break;
                    }
                }
                ChatResponse chatResponse  = parseResponse(fullResponse.toString(),requestDTO);
                emitter.send(chatResponse);
                emitter.complete();

            }

            @Override
            public void onFailure(@NotNull Call call, @NotNull IOException e) {
                sendError(emitter, "Connection failed: " + e.getMessage());
            }
        });
    }

    private void processChunk(SseEmitter emitter, OllamaChatResponse chunk) throws IOException {
        ChatStreamResponse chatStreamResponse = new ChatStreamResponse();
        chatStreamResponse.setContent(chunk.getMessage().getContent());
        chatStreamResponse.setDone(chunk.isDone());
        emitter.send(objectMapper.writeValueAsString(chatStreamResponse));
    }


    private void sendError(SseEmitter emitter, String message) {
        try {
            emitter.send(objectMapper.writeValueAsString(
                    new ChatResponse()));
            emitter.completeWithError(new IOException(message));
        } catch (JsonProcessingException e) {
            emitter.completeWithError(e);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
